"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getOpenReleasePRForVersion = getOpenReleasePRForVersion;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getOpenReleasePRForVersion(prs, version) {
  if (_lodash.default.isEmpty(prs)) {
    return {
      isReadyToMerge: false,
      reason: 'No open PRs present in this repo'
    };
  }

  const prsForVersion = _lodash.default.filter(prs, pr => _lodash.default.includes(pr.title, version));

  if (_lodash.default.isEmpty(prsForVersion)) {
    return {
      isReadyToMerge: false,
      reason: `No PRs found for version ${version}. Make sure PR title is correct`
    };
  }

  const prsWithReleaseLabel = _lodash.default.filter(prsForVersion, pr => hasLabel(pr, 'release'));

  if (_lodash.default.isEmpty(prsWithReleaseLabel)) {
    return {
      isReadyToMerge: false,
      reason: 'No release label present'
    };
  }

  const pr = _lodash.default.first(prsWithReleaseLabel);

  if (!hasApproves(pr)) {
    return {
      isReadyToMerge: false,
      reason: `Release PR has no approves`
    };
  }

  if (hasLabel(pr, `don't-merge`)) {
    return {
      isReadyToMerge: false,
      reason: `Release PR has a "don't merge" label`
    };
  }

  if (!isTargetBranchMaster(pr)) {
    return {
      isReadyToMerge: false,
      reason: `Release PR is not targeted to master branch`
    };
  }

  if (!isFromReleaseBranch(pr)) {
    return {
      isReadyToMerge: false,
      reason: `Release PR is not originated from release/hotfix branch`
    };
  }

  if (!isMergeable(pr)) {
    return {
      isReadyToMerge: false,
      reason: `Release PR has merge conflicts`
    };
  }

  if (!canUserMerge(pr)) {
    return {
      isReadyToMerge: false,
      reason: `You don't have permissions to merge this PR. Ask someone why`
    };
  }

  return {
    isReadyToMerge: true,
    prTitle: pr.title,
    prNumber: pr.number
  };
}

function hasLabel(pr, label) {
  const labels = getPRLabels(pr);
  return _lodash.default.includes(labels, label);
}

function getPRLabels(pr) {
  return _lodash.default.map(_lodash.default.get(pr, 'labels.nodes'), 'name');
}

function hasApproves(pr) {
  const approves = _lodash.default.get(pr, 'reviews.nodes', []);

  return !_lodash.default.isEmpty(approves);
}

function isTargetBranchMaster(pr) {
  return pr.baseRefName === 'master';
}

function isFromReleaseBranch(pr) {
  return _lodash.default.startsWith(pr.headRefName, 'release/v') || _lodash.default.startsWith(pr.headRefName, 'hotfix/v');
}

function isMergeable(pr) {
  return pr.mergeable === 'MERGEABLE';
}

function canUserMerge(pr) {
  return pr.viewerCanUpdate;
}